function [ssFea, msFea] = getFeatureVectors( magMat, orMat, nBins )

% This function computes the feature vectors for the interaction
% and association potentials of an image.  The values are stored
% in cell array indexed by the block sites.
%
% function [ssFea, msFea] = getFeatureVectors( magMat, orMat )
%
%
% parameters:
%   INPUTS:
%	magMat - the gradient intensity matrix for an image
%	orMat - the matrix of gradient orientations
%	nBins - the number of bins to use in histograms.  Default 50
%
%  OUTPUTS:
%	ssFea - A cell array.  The ith component is the 
%		ith sites singlescale feature vector
%       msFea - A cell array.  The ith component is the
%		ith sites multiscale feature vector.
%  Feature vectors are COLUMN vectors
%
% Author: Tim Rees
% Date: Nov 20, 2004
%
% 

%---------------------------------------------------------------------------

% Set up the image sites.

   bWid = 16;  %the block width
   bLen = 16; %the block length

   imWid = size( magMat,2);
   imLen = size( magMat,1);

%---------------------------------------------------------------------------

% Set up the histogram sizing
   if(nargin <3)
      nBins = 50; %roughly 5 degrees per bin
   end

% Compute the bin centers: they don't change
   x=2*pi/nBins;
   binCents = -pi+x:x:pi;
   binCents = binCents';

   nScales = 3; % the number of scales to use 

%----------------------------------------------------------------------------

% loop through block by block and compute the feature vector

   % keep track of the site index
   currSite = 1;
   row = 1;
   column = 1;

for j=1:bLen:imLen  

   % reset to first column
   column = 1; 
   for i=1:bWid:imWid	
	
      for scale=1:nScales
	
	% get the dimensions of the site at this scale
	[minx, maxx, miny, maxy] = getBounds(i,j,scale,imLen,imWid,bWid);

	% extract the grad mag of each block
	blockMag = magMat( miny:maxy, minx:maxx);
	% convert to a vector
	blockMag = blockMag(:);

	% extract the grad orientation of the block
	blockOr = orMat(miny:maxy, minx:maxx);
	% convert to a vector
	blockOr = blockOr(:);

	% compute histogram of the block of orientations,
	% add gradient magnitude to bins instead of count 1
	wCounts = weightedHist(blockOr, blockMag, binCents );

	% kernel smooth the weighted histogram
	smoothCounts = kernelSmoother( wCounts );
		
	% smoothCounts now contains the smoothed hist
	% extract features from it
	
	% get the moment based features	
	moms = getMoments( smoothCounts, 2);

	% get the orientation based features
	[ orients(1),orients(2)]=getOrientFeatures(smoothCounts, binCents);
	
	% fill the single and multi-scale features features
	
	%interFeatures{currSite} = ifv;


	
	fv{scale}(1:3) = moms;
	fv{scale}(4:5) = orients;
	  
	
     end % increase scale
         
     ssFea{row, column} = fv{1}';
     msFea{row, column}(1:5) = fv{1};
     msFea{row, column}(6:10) = fv{2};
     msFe{row,column}(5) = msFea{row,column}(5) -fv{2}(5);
     msFea{row,column}(10) = fv{2}(5) -fv{3}(5);
     msFea{row, column}(11:14) = fv{3}(1:4);

     msFea{row, column} = msFea{row,column}';
    
	
     column = column + 1;
     currSite = currSite + 1;
   end % step one horizontal block
   row = row + 1;
end % step one vertical block



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% in-line supporting functions

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function moment = getMoments( smCnts,numMom)

% computes the heaved center shifted cunts for the smoothed histogram
% count vector smCnts
% the variable p will determine which moment we are taking.


  moment(1) = mean(smCnts);
  v0=moment(1);

  for p=1:numMom

   H=smCnts>v0;
   diffVec = smCnts - v0;

   % compute the denominator
   denom = diffVec.*H;
   denom=sum(denom(:));

   %consider the special case when the weighted counts are zero
   % This means there is orientation 0 and magnitude 0 for image
   % gradient
   if( denom == 0 )
	
	moment(p+1) = 0;
   else

      diffVec=diffVec.^(p+1);

      % compute the numerator
      numer = diffVec.*H;
      numer=sum(numer(:));

      moment(p+1) = numer/denom;
    end
  end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%-------------------------------------------------------------------------
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [max_loc, orient] = getOrientFeatures( smCnts, centers )

% computes the orientation based feature for the smoothed histogram
% note smCnts>=0 at each index

% find the maxima over the smooth counts


   [maxval, maxind] = max(smCnts);

   % consider the case when smCnts is 0 identically
   if( maxval ==0)
	orient = 0;
	max_loc = 0;
   else

   smCnts( maxind) = -1;

      [maxval2, maxind2] = max(smCnts);

      % get the sinusoidal difference of the distance
      % between the maxima
      orient = abs( sin( centers(maxind) - centers(maxind2) ));

      % get the location of the maxima (in radians)

      max_loc = centers(maxind);

   end


